@@ -1,5 +1,5 @@
 /*
- *	$Id: lspci.c,v 1.11 1998/04/19 11:02:27 mj Exp $
+ *	$Id: lspci.c,v 1.17 1998/11/18 14:06:07 mj Exp $
  *
  *	Linux PCI Utilities -- List All PCI Devices
  *
@@ -34,7 +34,7 @@ Usage: lspci [<switches>]\n\
 -v\t\tBe verbose\n\
 -n\t\tShow numeric ID's\n\
 -b\t\tBus-centric view (PCI addresses and IRQ's instead of those seen by the CPU)\n\
--x\t\tShow hex-dump of config space (-xx shows full 256 bytes)\n\
+-x\t\tShow hex-dump of config space\n\
 -s [[<bus>]:][<slot>][.[<func>]]\tShow only devices in selected slots\n\
 -d [<vendor>]:[<device>]\tShow only selected devices\n\
 -t\t\tShow bus tree\n\
@@ -137,7 +137,7 @@ scan_config(void)
   struct device *d;
   char name[64];
   int fd, res;
-  int how_much = (show_hex > 1) ? 256 : 64;
+  int how_much = (show_hex > 2) ? 256 : 64;
 
   for(d=first_dev; d; d=d->next)
     {
@@ -262,53 +262,72 @@ show_bases(struct device *d, int cnt)
   word cmd = get_conf_word(d, PCI_COMMAND);
   int i;
 
-  for(i=0; i<6; i++)
+  for(i=0; i<cnt; i++)
     {
       unsigned long pos;
       unsigned int flg = get_conf_long(d, PCI_BASE_ADDRESS_0 + 4*i);
       if (buscentric_view)
 	pos = flg;
       else
-	pos = d->kernel_base_addr[i];
-      if (!pos || pos == 0xffffffff)
+	{
+	  pos = d->kernel_base_addr[i];
+	  if (!pos)
+	    continue;
+	}
+      if (pos == 0xffffffff)
 	continue;
+      if (verbose > 1)
+	printf("\tRegion %d: ", i);
+      else
+	putchar('\t');
       if (flg & PCI_BASE_ADDRESS_SPACE_IO)
 	{
-	  if (cmd & PCI_COMMAND_IO)
-	    {
-	      if (verbose > 1)
-		printf("\tRegion %d: ", i);
-	      else
-		putchar('\t');
-	      printf("I/O ports at %04lx\n", pos & PCI_BASE_ADDRESS_IO_MASK);
-	    }
+	  unsigned long a = pos & PCI_BASE_ADDRESS_IO_MASK;
+	  printf("I/O ports at ");
+	  if (a)
+	    printf("%04lx", a);
+	  else
+	    printf("<unassigned>");
+	  if (!(cmd & PCI_COMMAND_IO))
+	    printf(" [disabled]");
 	}
-      else if (cmd & PCI_COMMAND_MEMORY)
+      else
 	{
 	  int t = flg & PCI_BASE_ADDRESS_MEM_TYPE_MASK;
-	  if (verbose > 1)
-	    printf("\tRegion %d: ", i);
-	  else
-	    putchar('\t');
+	  unsigned long a = pos & PCI_BASE_ADDRESS_MEM_MASK;
+	  int x64 = 0;
 	  printf("Memory at ");
 	  if (t == PCI_BASE_ADDRESS_MEM_TYPE_64)
 	    {
 	      if (i < cnt - 1)
 		{
+		  u32 z;
 		  i++;
-		  if (!buscentric_view)
-		    printf("%08x", get_conf_long(d, PCI_BASE_ADDRESS_0 + 4*i));
+		  z = get_conf_long(d, PCI_BASE_ADDRESS_0 + 4*i);
+		  if (buscentric_view)
+		    printf("%08x", z);
+		  if (z)
+		    x64 = 1;
 		}
 	      else
-		printf("????????");
+		{
+		  printf("????????");
+		  x64 = 1;
+		}
 	    }
-	  printf("%08lx (%s, %sprefetchable)\n",
-		 pos & PCI_BASE_ADDRESS_MEM_MASK,
+	  if (x64 || a)
+	    printf("%08lx", a);
+	  else
+	    printf("<unassigned>");
+	  printf(" (%s, %sprefetchable)",
 		 (t == PCI_BASE_ADDRESS_MEM_TYPE_32) ? "32-bit" :
 		 (t == PCI_BASE_ADDRESS_MEM_TYPE_64) ? "64-bit" :
-		 (t == PCI_BASE_ADDRESS_MEM_TYPE_1M) ? "low-1M 32-bit" : "???",
+		 (t == PCI_BASE_ADDRESS_MEM_TYPE_1M) ? "low-1M" : "type 3",
 		 (flg & PCI_BASE_ADDRESS_MEM_PREFETCH) ? "" : "non-");
+	  if (!(cmd & PCI_COMMAND_MEMORY))
+	    printf(" [disabled]");
 	}
+      putchar('\n');
     }
 }
 
@@ -380,7 +399,7 @@ show_htype1(struct device *d)
       pref_base = (pref_base & PCI_PREF_RANGE_MASK) << 16;
       pref_limit = (pref_limit & PCI_PREF_RANGE_MASK) << 16;
       if (pref_type == PCI_PREF_RANGE_TYPE_32)
-	printf("\tPrefetchable memory behind bridge: %08x-%08x\n", pref_base, pref_limit);
+	printf("\tPrefetchable memory behind bridge: %08x-%08x\n", pref_base, pref_limit + 0xfffff);
       else
 	printf("\tPrefetchable memory behind bridge: %08x%08x-%08x%08x\n",
 	       get_conf_long(d, PCI_PREF_BASE_UPPER32),
@@ -410,6 +429,59 @@ show_htype1(struct device *d)
 static void
 show_htype2(struct device *d)
 {
+  int i;
+  word cmd = get_conf_word(d, PCI_COMMAND);
+  word brc = get_conf_word(d, PCI_CB_BRIDGE_CONTROL);
+  word exca = get_conf_word(d, PCI_CB_LEGACY_MODE_BASE);
+
+  show_bases(d, 1);
+  printf("\tBus: primary=%02x, secondary=%02x, subordinate=%02x, sec-latency=%d\n",
+	 get_conf_byte(d, PCI_CB_PRIMARY_BUS),
+	 get_conf_byte(d, PCI_CB_CARD_BUS),
+	 get_conf_byte(d, PCI_CB_SUBORDINATE_BUS),
+	 get_conf_byte(d, PCI_CB_LATENCY_TIMER));
+  for(i=0; i<2; i++)
+    {
+      int p = 8*i;
+      u32 base = get_conf_long(d, PCI_CB_MEMORY_BASE_0 + p);
+      u32 limit = get_conf_long(d, PCI_CB_MEMORY_LIMIT_0 + p);
+      if (limit > base)
+	printf("Memory window %d: %08x-%08x%s%s\n", i, base, limit,
+	       (cmd & PCI_COMMAND_MEMORY) ? "" : " [disabled]",
+	       (brc & (PCI_CB_BRIDGE_CTL_PREFETCH_MEM0 << i)) ? " (prefetchable)" : "");
+    }
+  for(i=0; i<2; i++)
+    {
+      int p = 8*i;
+      u32 base = get_conf_long(d, PCI_CB_IO_BASE_0 + p);
+      u32 limit = get_conf_long(d, PCI_CB_IO_LIMIT_0 + p);
+      if (!(base & PCI_IO_RANGE_TYPE_32))
+	{
+	  base &= 0xffff;
+	  limit &= 0xffff;
+	}
+      base &= PCI_CB_IO_RANGE_MASK;
+      if (!base)
+	continue;
+      limit = (limit & PCI_CB_IO_RANGE_MASK) + 3;
+      printf("I/O window %d: %08x-%08x%s\n", i, base, limit,
+	     (cmd & PCI_COMMAND_IO) ? "" : " [disabled]");
+    }
+
+  if (get_conf_word(d, PCI_CB_SEC_STATUS) & PCI_STATUS_SIG_SYSTEM_ERROR)
+    printf("\tSecondary status: SERR\n");
+  if (verbose > 1)
+    printf("\tBridgeCtl: Parity%c SERR%c ISA%c VGA%c MAbort%c >Reset%c 16bInt%c PostWrite%c\n",
+	   (brc & PCI_CB_BRIDGE_CTL_PARITY) ? '+' : '-',
+	   (brc & PCI_CB_BRIDGE_CTL_SERR) ? '+' : '-',
+	   (brc & PCI_CB_BRIDGE_CTL_ISA) ? '+' : '-',
+	   (brc & PCI_CB_BRIDGE_CTL_VGA) ? '+' : '-',
+	   (brc & PCI_CB_BRIDGE_CTL_MASTER_ABORT) ? '+' : '-',
+	   (brc & PCI_CB_BRIDGE_CTL_CB_RESET) ? '+' : '-',
+	   (brc & PCI_CB_BRIDGE_CTL_16BIT_INT) ? '+' : '-',
+	   (brc & PCI_CB_BRIDGE_CTL_POST_WRITES) ? '+' : '-');
+  if (exca)
+    printf("\t16-bit legacy interface ports at %04x\n", exca);
 }
 
 static void
@@ -453,8 +525,9 @@ show_verbose(struct device *d)
     case PCI_HEADER_TYPE_CARDBUS:
       if ((class >> 8) != PCI_BASE_CLASS_BRIDGE)
 	goto badhdr;
-      irq = int_line = int_pin = min_gnt = max_lat = 0;
-      subsys_v = subsys_d = 0;
+      min_gnt = max_lat = 0;
+      subsys_v = get_conf_word(d, PCI_CB_SUBSYSTEM_VENDOR_ID);
+      subsys_d = get_conf_word(d, PCI_CB_SUBSYSTEM_ID);
       break;
     default:
       printf("\t!!! Unknown header type %02x\n", htype);
@@ -466,10 +539,11 @@ show_verbose(struct device *d)
   else
     irq = d->kernel_irq;
 
+  if (verbose && subsys_v && subsys_v != 0xffff)
+    printf("\tSubsystem: %s\n", lookup_subsys_device_full(subsys_v, subsys_d));
+
   if (verbose > 1)
     {
-      if (subsys_v)
-	printf("\tSubsystem ID: %04x:%04x\n", subsys_v, subsys_d);
       printf("\tControl: I/O%c Mem%c BusMaster%c SpecCycle%c MemWINV%c VGASnoop%c ParErr%c Stepping%c SERR%c FastB2B%c\n",
 	     (cmd & PCI_COMMAND_IO) ? '+' : '-',
 	     (cmd & PCI_COMMAND_MEMORY) ? '+' : '-',
@@ -564,7 +638,7 @@ static void
 show_hex_dump(struct device *d)
 {
   int i;
-  int limit = (show_hex > 1) ? 256 : 64;
+  int limit = (show_hex > 2) ? 256 : 64;
 
   for(i=0; i<limit; i++)
     {
@@ -580,6 +654,19 @@ static void
 show_machine(struct device *d)
 {
   int c;
+  word sv_id=0, sd_id=0;
+
+  switch (get_conf_byte(d, PCI_HEADER_TYPE) & 0x7f)
+    {
+    case PCI_HEADER_TYPE_NORMAL:
+      sv_id = get_conf_word(d, PCI_SUBSYSTEM_VENDOR_ID);
+      sd_id = get_conf_word(d, PCI_SUBSYSTEM_ID);
+      break;
+    case PCI_HEADER_TYPE_CARDBUS:
+      sv_id = get_conf_word(d, PCI_CB_SUBSYSTEM_VENDOR_ID);
+      sd_id = get_conf_word(d, PCI_CB_SUBSYSTEM_ID);
+      break;
+    }
 
   if (verbose)
     {
@@ -587,6 +674,11 @@ show_machine(struct device *d)
       printf("Class:\t%s\n", lookup_class(get_conf_word(d, PCI_CLASS_DEVICE)));
       printf("Vendor:\t%s\n", lookup_vendor(d->vendid));
       printf("Device:\t%s\n", lookup_device(d->vendid, d->devid));
+      if (sv_id && sv_id != 0xffff)
+	{
+	  printf("SVendor:\t%s\n", lookup_subsys_vendor(sv_id));
+	  printf("SDevice:\t%s\n", lookup_subsys_device(sv_id, sd_id));
+	}
       if (c = get_conf_byte(d, PCI_REVISION_ID))
 	printf("Rev:\t%02x\n", c);
       if (c = get_conf_byte(d, PCI_CLASS_PROG))
@@ -603,6 +695,10 @@ show_machine(struct device *d)
 	printf(" -r%02x", c);
       if (c = get_conf_byte(d, PCI_CLASS_PROG))
 	printf(" -p%02x", c);
+      if (sv_id && sv_id != 0xffff)
+	printf(" \"%s\" \"%s\"", lookup_subsys_vendor(sv_id), lookup_subsys_device(sv_id, sd_id));
+      else
+	printf(" \"\" \"\"");
       putchar('\n');
     }
 }
@@ -704,12 +800,23 @@ grow_tree(void)
   for(d=first_dev; d; d=d->next)
     {
       word class = get_conf_word(d, PCI_CLASS_DEVICE);
-      if (class == PCI_CLASS_BRIDGE_PCI && (get_conf_byte(d, PCI_HEADER_TYPE) & 0x7f) == 1)
+      byte ht = get_conf_byte(d, PCI_HEADER_TYPE) & 0x7f;
+      if (class == PCI_CLASS_BRIDGE_PCI &&
+	  (ht == PCI_HEADER_TYPE_BRIDGE || ht == PCI_HEADER_TYPE_CARDBUS))
 	{
 	  b = xmalloc(sizeof(struct bridge));
-	  b->primary = get_conf_byte(d, PCI_PRIMARY_BUS);
-	  b->secondary = get_conf_byte(d, PCI_SECONDARY_BUS);
-	  b->subordinate = get_conf_byte(d, PCI_SUBORDINATE_BUS);
+	  if (ht == PCI_HEADER_TYPE_BRIDGE)
+	    {
+	      b->primary = get_conf_byte(d, PCI_CB_PRIMARY_BUS);
+	      b->secondary = get_conf_byte(d, PCI_CB_CARD_BUS);
+	      b->subordinate = get_conf_byte(d, PCI_CB_SUBORDINATE_BUS);
+	    }
+	  else
+	    {
+	      b->primary = get_conf_byte(d, PCI_PRIMARY_BUS);
+	      b->secondary = get_conf_byte(d, PCI_SECONDARY_BUS);
+	      b->subordinate = get_conf_byte(d, PCI_SUBORDINATE_BUS);
+	    }
 	  *last_br = b;
 	  last_br = &b->chain;
 	  b->next = b->child = NULL;
