@@ -1,5 +1,5 @@
 /*
- *	$Id: pciutils.h,v 1.6 1998/06/12 09:48:36 mj Exp $
+ *	$Id: pciutils.h,v 1.9 1998/10/19 13:40:45 mj Exp $
  *
  *	Linux PCI Utilities -- Declarations
  *
@@ -16,10 +16,10 @@
 #include "pci.h"
 #endif
 
-#define PCIUTILS_VERSION "1.06"
+#define PCIUTILS_VERSION "1.09"
 
 #define PROC_BUS_PCI "/proc/bus/pci"
-#define ETC_PCI_IDS "/etc/pci.ids"
+#define ETC_PCI_IDS "/usr/share/pci.ids"
 
 /* Types */
 
@@ -40,6 +40,9 @@ char *lookup_vendor(word);
 char *lookup_device(word, word);
 char *lookup_device_full(word, word);
 char *lookup_class(word);
+char *lookup_subsys_vendor(word);
+char *lookup_subsys_device(word, word);
+char *lookup_subsys_device_full(word, word);
 
 /* filter.c */
 
